   // Converts the Chromatic Number problem to HCP.
   //
   // Input should be the desired number of colours, and COL file describing the graph, in the following format:
   //
   // c COMMENTS
   // c COMMENTS
   // p edge <number of vertices> <number of edges>
   // e 1 2
   // e 1 5
   // e 1 6
   // e 2 1
   // ...
   // e 10 8
   //
   //
   // Note that the .col file MUST have edges listed in both directions!
   //
   // Written by Michael Haythorpe
   
   import java.io.*;
   import java.util.StringTokenizer;
   import java.util.Arrays;

   public class COL2HCP
   {
      public static void main(String [] args)
      {
         if(args.length < 2 || args.length > 3)
         {
            System.out.println("COL2HCP should be run using the following input:");
            System.out.println();
            System.out.println("COL2HCP.exe <size of desired colouring> <input_file.col> <output_file.hcp>");
            System.out.println();
            System.out.println("If no output filename is specified, COL2HCP creates input_file.hcp.");
            return;
         }
      
         String inputFilename = "";
         String outputFilename = "";
         int k = 0;
         int literals = 0;
         int clauses = 0;
      
         if(args.length == 2)
         {
            try
            {
               k = Integer.parseInt(args[0]);
            }
               catch(Exception e)
               {
                  System.out.println("Colouring size should be an integer, rather than " + args[0] + ".");
                  return;
               }
            inputFilename = args[1];
            outputFilename = inputFilename;
            if(outputFilename.substring(outputFilename.length()-4).equals(".col"))
               outputFilename = outputFilename.substring(0,outputFilename.length()-4);
            outputFilename += ".hcp";
         }
      
         if(args.length == 3)
         {
            try
            {
               k = Integer.parseInt(args[0]);
            }
               catch(Exception e)
               {
                  System.out.println("Colouring size should be an integer, rather than " + args[0] + ".");
                  return;
               }
            inputFilename = args[1];
            outputFilename = args[2];
         }
      
         File inputFile = new File(inputFilename);
         File outputFile = new File(outputFilename);
         
         int origN = 0;
         int origEdges = 0;
      	
         try
         {
            BufferedReader br = new BufferedReader(new FileReader(inputFile));
            String line = br.readLine();
            int [][]edges = null;
            int count = 0;
            while(line != null)
            {
               if(line.charAt(0) != 'c')
               {
                  if(line.charAt(0) == 'p')
                  {
                     StringTokenizer tokens = new StringTokenizer(line);
                     tokens.nextToken();
                     if(!tokens.nextToken().equals("edge"))
                     {
                        System.out.println(inputFilename + " is not in edge-list format!");
                        return;
                     }
                     origN = Integer.parseInt(tokens.nextToken());
                     origEdges = Integer.parseInt(tokens.nextToken())/2;
                     edges = new int[origEdges][2];
                  }
                  else
                  {
                     StringTokenizer tokens = new StringTokenizer(line);
                     tokens.nextToken();
                     int first = Integer.parseInt(tokens.nextToken());
                     int second = Integer.parseInt(tokens.nextToken());
                  	
                     if(first < second)
                     {
                        edges[count][0] = first;
                        edges[count][1] = second;
                        count++;
                     }
                    
                  }
               }
               line = br.readLine();
            }
          
         
            int N = 2*k*origN + (2+6*k)*origEdges + 2;
            int numberEdges = 3*k*origN + (k*k+11*k+1)*origEdges + 2;
         
            int [][]newEdges = new int[numberEdges][2];
            
         	/*
         	 Vertex breakdown:
         	 0, N-1 : S & F
         	 2N vertice for each colour
         	 (2+6*k) vertices for each edge		
         	*/
         
            int edgeCount = 0;
         
            for(int i=0; i<=2*origN*k; i++)
            {
               newEdges[edgeCount][0] = i;
               newEdges[edgeCount][1] = i+1;
               edgeCount++;
               
            }
         
         
            for(int i=0; i<origEdges; i++)
            {
               for(int j=0; j<k; j++)
               {
                  newEdges[edgeCount][0] = 2*k*origN + (2+6*k)*i + 1;
                  newEdges[edgeCount][1] = 2*k*origN + (2+6*k)*i + 2+3*j;
                  edgeCount++;
               }
               
               for(int j=0; j<2*k; j++)
               {
                  newEdges[edgeCount][0] = 2*k*origN + (2+6*k)*i + 2+3*j;
                  newEdges[edgeCount][1] = 2*k*origN + (2+6*k)*i + 2+3*j + 1;
                  edgeCount++;
               	
                  newEdges[edgeCount][0] = 2*k*origN + (2+6*k)*i + 2+3*j + 1;
                  newEdges[edgeCount][1] = 2*k*origN + (2+6*k)*i + 2+3*j;
                  edgeCount++;
               	
                  newEdges[edgeCount][0] = 2*k*origN + (2+6*k)*i + 2+3*j + 1;
                  newEdges[edgeCount][1] = 2*k*origN + (2+6*k)*i + 2+3*j + 2;
                  edgeCount++;
               	
                  newEdges[edgeCount][0] = 2*k*origN + (2+6*k)*i + 2+3*j + 2;
                  newEdges[edgeCount][1] = 2*k*origN + (2+6*k)*i + 2+3*j + 1;
                  edgeCount++;
               
               }
               
               for(int j=0; j<k; j++)
               {
                  newEdges[edgeCount][0] = 2*k*origN + (2+6*k)*i + 2+3*(k+j) + 2;
                  newEdges[edgeCount][1] = 2*k*origN + (2+6*k)*(i+1);
                  edgeCount++;
               }
               
               for(int j=0; j<k; j++)
                  for(int l=0; l<k; l++)
                  {
                     if(j != l)
                     {
                        newEdges[edgeCount][0] = 2*k*origN + (2+6*k)*i + 2+3*j + 2;
                        newEdges[edgeCount][1] = 2*k*origN + (2+6*k)*i + 2+3*(k+l);
                        edgeCount++;
                     }
                  }
            	
               newEdges[edgeCount][0] = 2*k*origN + (2+6*k)*(i+1);
               newEdges[edgeCount][1] = 2*k*origN + (2+6*k)*(i+1) + 1;
               edgeCount++;
            	
            }
         
            
            newEdges[edgeCount][0] = N-1;
            newEdges[edgeCount][1] = 0;
            edgeCount++;
         	// Now to add additional edges
         
            for(int i=0; i<k; i++)   
            {
               
               for(int j=0; j<origN; j++)
               {
                  int current = 1+2*j+2*i*origN;
                  for(int l=0; l<origEdges; l++)
                  {
                     if(edges[l][0] == j+1)
                     {
                        newEdges[edgeCount][0] = current;
                        newEdges[edgeCount][1] = 2*k*origN + (2+6*k)*l + 4+3*i;
                        current = 2*k*origN + (2+6*k)*l + 4+3*i - 2;
                        edgeCount++;
                     }
                     else
                     {
                        if(edges[l][1] == j+1)
                        {
                           newEdges[edgeCount][0] = current;
                           newEdges[edgeCount][1] = 2*k*origN + (2+6*k)*l + 4+3*(i+k);
                           current = 2*k*origN + (2+6*k)*l + 4+3*(i+k) - 2;
                           edgeCount++;
                        
                        }
                     }
                  
                  }
                  
                  if(current != 1+2*j+2*i*origN)
                  {
                     newEdges[edgeCount][0] = current;
                     newEdges[edgeCount][1] = 2+2*j+2*i*origN;
                     edgeCount++;
                  }
                                      
               }
            }
         
         
            
            /*for(int i=0; i<origEdges; i++)
            {
               for(int j=0; j<k; j++)
               {
                  newEdges[edgeCount][0] = 2*k*origN + (k*k+11*k+1)*i + 1;
                  newEdges[edgeCount][1] = 2*k*origN + (k*k+11*k+1)*i + 2+3*j;
                  edgeCount++;
               }
               
               for(int j=0; j<2*k; j++)
               {
                  newEdges[edgeCount][0] = 2*k*origN + (k*k+11*k+1)*i + 2+3*j;
                  newEdges[edgeCount][1] = 2*k*origN + (k*k+11*k+1)*i + 2+3*j + 1;
                  edgeCount++;
               	
                  newEdges[edgeCount][0] = 2*k*origN + (k*k+11*k+1)*i + 2+3*j + 1;
                  newEdges[edgeCount][1] = 2*k*origN + (k*k+11*k+1)*i + 2+3*j;
                  edgeCount++;
               	
                  newEdges[edgeCount][0] = 2*k*origN + (k*k+11*k+1)*i + 2+3*j + 1;
                  newEdges[edgeCount][1] = 2*k*origN + (k*k+11*k+1)*i + 2+3*j + 2;
                  edgeCount++;
               	
                  newEdges[edgeCount][0] = 2*k*origN + (k*k+11*k+1)*i + 2+3*j + 2;
                  newEdges[edgeCount][1] = 2*k*origN + (k*k+11*k+1)*i + 2+3*j + 1;
         			edgeCount++;
         
               }
               
               for(int j=0; j<k; j++)
               {
                  newEdges[edgeCount][0] = 2*k*origN + (k*k+11*k+1)*i + 2+3*(k+j) + 2;
                  newEdges[edgeCount][1] = 2*k*origN + (k*k+11*k+1)*(i+1);
                  edgeCount++;
               }
               
               for(int j=0; j<k; j++)
                  for(int l=0; l<k; l++)
                  {
                     if(j != l)
                     {
                        newEdges[edgeCount][0] = 2*k*origN + (k*k+11*k+1)*i + 2+3*j + 2;
                        newEdges[edgeCount][1] = 2*k*origN + (k*k+11*k+1)*i + 2+3*(k+l);
                        edgeCount++;
                     }
                  }
            	
               newEdges[edgeCount][0] = 2*k*origN + (k*k+11*k+1)*(i+1);
               newEdges[edgeCount][1] = 2*k*origN + (k*k+11*k+1)*(i+1) + 1;
               edgeCount++;
            	
            }
         
            
            newEdges[edgeCount][0] = numberEdges-1;
            newEdges[edgeCount][1] = 0;
            edgeCount++;
         	// Now to add additional edges
         
            for(int i=0; i<k; i++)   
            {
               
               for(int j=0; j<origN; j++)
               {
                  int current = 1+2*j+2*i*origN;
                  for(int l=0; l<origEdges; l++)
                  {
                     if(edges[l][0] == j)
                     {
                        newEdges[edgeCount][0] = current;
                        newEdges[edgeCount][1] = 2*k*origN + (k*k+11*k+1)*l + 4+3*i;
                        current = 2*k*origN + (k*k+11*k+1)*l + 4+3*i - 2;
                        edgeCount++;
                     }
                     else
                     {
                        if(edges[l][1] == j)
                        {
                           newEdges[edgeCount][0] = current;
                           newEdges[edgeCount][1] = 2*k*origN + (k*k+11*k+1)*l + 4+3*(i+k);
                           current = 2*k*origN + (k*k+11*k+1)*l + 4+3*(i+k) - 2;
                           edgeCount++;
                        
                        }
                     }
                  
                  }
                  
                  if(current != 1+2*j+2*i*origN)
                  {
                     newEdges[edgeCount][0] = current;
                     newEdges[edgeCount][1] = 2+2*j+2*i*origN;
                     edgeCount++;
                  }
                                      
               }
            }*/
            
         
            
            int [][]finalEdges = new int[3*N][k+1];
            int []edgeCounts = new int[3*N];
         
            for(int i=0; i<N; i++)
            {
               finalEdges[3*i][edgeCounts[3*i]++] = 3*i+2;
               finalEdges[3*i+1][edgeCounts[3*i+1]++] = 3*i+1;
               finalEdges[3*i+1][edgeCounts[3*i+1]++] = 3*i+3;
               finalEdges[3*i+2][edgeCounts[3*i+2]++] = 3*i+2;
            }
         	        
            for(int i=0; i<edgeCount; i++)
            {
            /*System.out.println("N : " + N);
            System.out.println("edgeCount : " + edgeCount);
            System.out.println("Processing edge " + i + " : " + newEdges[i][0] + " , " + newEdges[i][1]);
               System.out.println("Top: " + (newEdges[i][0]*3+2));
               System.out.println("Top2 : " + (edgeCounts[newEdges[i][0]*3+2]));
               System.out.println("TopRHS : " + (newEdges[i][1]*3+1));
               System.out.println("Bottom : " + newEdges[i][1]*3);
               System.out.println("Bottom2 : " + edgeCounts[newEdges[i][1]*3]);
               System.out.println("BottomLHS : " + (newEdges[i][0]*3+3));
            	System.out.println(" ");*/
               finalEdges[newEdges[i][0]*3+2][edgeCounts[newEdges[i][0]*3+2]++] = newEdges[i][1]*3+1;
               finalEdges[newEdges[i][1]*3][edgeCounts[newEdges[i][1]*3]++] = newEdges[i][0]*3+3;
              
            	
            	
            }
            
            for(int i=0; i<3*N; i++)
               for(int j=0; j<k+1; j++)
                  if(finalEdges[i][j] == 0)
                     finalEdges[i][j] = 3*N+1;
            
         
            
            for(int i=0; i<3*N; i++)
            {
               Arrays.sort(finalEdges[i]);
            }
            
            for(int i=0; i<3*N; i++)
               for(int j=0; j<k+1; j++)
                  if(finalEdges[i][j] == 3*N+1)
                     finalEdges[i][j] = 0;
              
            BufferedWriter bw = new BufferedWriter(new FileWriter(outputFile));
            
            bw.write("NAME : Adapted from COL file " + inputFilename);
            bw.newLine();
            bw.write("COMMENT : ");
            bw.newLine();
            bw.write("TYPE : HCP");
            bw.newLine();
            bw.write("DIMENSION : " + 3*N);
            bw.newLine();
            bw.write("EDGE_DATA_FORMAT : EDGE_LIST");
            bw.newLine();
            bw.write("EDGE_DATA_SECTION");
            bw.newLine();
         
            for(int i=0; i<3*N; i++)
               for(int j=0; j<finalEdges[i].length; j++)
               {
                  if(finalEdges[i][j] == 0)
                     break;
               
                  bw.write((i+1) + " " + finalEdges[i][j]);
                  bw.newLine();
               }
         
            bw.write("-1");
            bw.newLine();
            bw.write("EOF");
            bw.close();
         
         
         }
            catch(Exception e)
            {
               System.out.println(inputFilename + " is not in edge-list format!");
               return;
            }
      	
      }
   }
